import { ElementCreator } from './elementCreator';
import { Cell, CodeCell, DisplayData, Error as NbError, ExecuteResult, MarkdownCell, Notebook, Output, RawCell, Stream as NbStream } from './nbformat';
export declare type Options<TElement = HTMLElement> = {
    /**
     * An object with additional data renderers indexed by a media type.
     */
    dataRenderers?: DataRenderers<TElement>;
    /**
     * An array of the supported media types in the priority order. When a cell
     * contains multiple representations of the data, the one with the media type
     * that has the lowest index in this array will be rendered. The default is
     * `Object.keys({ ...dataRenderers, ...builtinRenderers })`.
     */
    dataRenderersOrder?: string[];
    /**
     * A function for converting ANSI escape sequences in the given *text* to HTML.
     * It gets the text from the cell as-is, without prior escaping, so it must
     * escape special characters unsafe for HTML (ansi_up does it implicitly)!
     */
    ansiCodesRenderer?: (text: string) => string;
    /**
     * A function for highlighting the given source *code*, it should return an
     * HTML string. It gets the text from the cell as-is, without prior escaping,
     * so it must escape special characters unsafe for HTML (highlight.js does it
     * implicitly)!
     */
    codeHighlighter?: (code: string, lang: string) => string;
    /**
     * A function for converting the given Markdown source to HTML.
     */
    markdownRenderer?: (markup: string) => string;
};
export declare type DataRenderer<TElement = HTMLElement> = (data: string) => TElement;
declare type DataRenderers<TElement> = {
    [mediaType: string]: DataRenderer<TElement>;
};
/**
 * Builds a Notebook renderer function with the given options. It returns a
 * "callable object" that exposes a renderer function for each of the
 * Notebook's AST node. You can easily replace any of the functions to modify
 * behaviour of the renderer.
 *
 * @param {ElementCreator} elementCreator The function that will be used for
 *   building all HTML elements.
 * @param {Options} opts
 * @return {NbRenderer}
 * @template TElement Type of the element object that *elementCreator* produces.
 */
declare function buildRenderer<TElement>(elementCreator: ElementCreator<TElement>, opts?: Options<TElement>): {
    Notebook: (notebook: Notebook) => TElement;
    Cell: (cell: Cell, notebook: Notebook) => TElement;
    MarkdownCell: (cell: MarkdownCell, _notebook: Notebook) => TElement;
    RawCell: (cell: RawCell, _notebook: Notebook) => TElement;
    CodeCell: (cell: CodeCell, notebook: Notebook) => TElement;
    Source: (cell: CodeCell, notebook: Notebook) => TElement;
    Output: (output: Output, cell: CodeCell) => TElement;
    DisplayData: (output: DisplayData) => TElement;
    ExecuteResult: (output: ExecuteResult) => TElement;
    Error: (error: NbError) => TElement;
    Stream: (stream: NbStream) => TElement;
} & ((notebook: Notebook) => TElement);
export default buildRenderer;
declare abstract class DummyClass<T> {
    renderer: {
        Notebook: (notebook: Notebook) => T;
        Cell: (cell: Cell, notebook: Notebook) => T;
        MarkdownCell: (cell: MarkdownCell, _notebook: Notebook) => T;
        RawCell: (cell: RawCell, _notebook: Notebook) => T;
        CodeCell: (cell: CodeCell, notebook: Notebook) => T;
        Source: (cell: CodeCell, notebook: Notebook) => T;
        Output: (output: Output, cell: CodeCell) => T;
        DisplayData: (output: DisplayData) => T;
        ExecuteResult: (output: ExecuteResult) => T;
        Error: (error: NbError) => T;
        Stream: (stream: NbStream) => T;
    } & ((notebook: Notebook) => T);
    abstract elementCreator(): ElementCreator<T>;
}
export declare type NbRenderer<TElement> = DummyClass<TElement>['renderer'];
//# sourceMappingURL=renderer.d.ts.map