"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const utils_1 = require("./internal/utils");
const nbformat_1 = require("./nbformat");
function joinText(text) {
    return Array.isArray(text) ? text.map(joinText).join('') : text;
}
function coalesceStreams(outputs) {
    if (!outputs.length) {
        return outputs;
    }
    let last = outputs[0];
    const newOutputs = [last];
    for (const output of outputs.slice(1)) {
        if (output.output_type === 'stream' && last.output_type === 'stream' && output.name === last.name) {
            last.text = last.text.concat(...output.text);
        }
        else {
            newOutputs.push(output);
            last = output;
        }
    }
    return newOutputs;
}
function executionCountAttrs({ execution_count: count }) {
    return count ? {
        'data-execution-count': String(count),
        // Only for backward compatibility with notebook.js.
        'data-prompt-number': String(count),
    } : {};
}
function notebookLanguage({ metadata: meta }) {
    return (meta.language_info && meta.language_info.name) || 'python';
}
/**
 * Builds a Notebook renderer function with the given options. It returns a
 * "callable object" that exposes a renderer function for each of the
 * Notebook's AST node. You can easily replace any of the functions to modify
 * behaviour of the renderer.
 *
 * @param {ElementCreator} elementCreator The function that will be used for
 *   building all HTML elements.
 * @param {Options} opts
 * @return {NbRenderer}
 * @template TElement Type of the element object that *elementCreator* produces.
 */
function buildRenderer(elementCreator, opts = {}) {
    const renderMarkdown = opts.markdownRenderer || utils_1.identity;
    const renderAnsiCodes = opts.ansiCodesRenderer || utils_1.escapeHTML;
    const highlightCode = opts.codeHighlighter || utils_1.escapeHTML;
    const el = elementCreator;
    const el2 = (tag, classes) => (data) => el(tag, classes, data);
    const embeddedImageEl = (format) => (data) => el('img', {
        class: 'image-output',
        src: `data:image/${format};base64,${data.replace(/\n/g, '')}`,
    });
    // opts.dataRenderers is intentionally included twice; to get the user's
    // provided renderers in the default dataRenderersOrder before the built-in
    // renderers and at the same time allow to override any built-in renderer.
    const dataRenderers = {
        ...opts.dataRenderers,
        'image/png': embeddedImageEl('png'),
        'image/jpeg': embeddedImageEl('jpeg'),
        'image/svg+xml': el2('div', ['svg-output']),
        'text/svg+xml': (data) => dataRenderers['image/svg+xml'](data),
        'text/html': el2('div', ['html-output']),
        'text/markdown': (data) => dataRenderers['text/html'](renderMarkdown(data)),
        'text/latex': el2('div', ['latex-output']),
        'application/javascript': el2('script', []),
        'text/plain': (data) => el('pre', ['text-output'], utils_1.escapeHTML(data)),
        ...opts.dataRenderers,
    };
    const dataRenderersOrder = opts.dataRenderersOrder || Object.keys(dataRenderers);
    const resolveDataType = (output) => {
        return dataRenderersOrder.find(type => output.data[type] && dataRenderers[type]);
    };
    const r = utils_1.callableObject('Notebook', {
        Notebook: (notebook) => {
            const children = notebook.cells.map(cell => r.Cell(cell, notebook));
            // Class "worksheet" is for backward compatibility with notebook.js.
            return el('div', ['notebook', 'worksheet'], children);
        },
        Cell: (cell, notebook) => {
            switch (cell.cell_type) {
                case nbformat_1.CellType.Code: return r.CodeCell(cell, notebook);
                case nbformat_1.CellType.Markdown: return r.MarkdownCell(cell, notebook);
                case nbformat_1.CellType.Raw: return r.RawCell(cell, notebook);
                default: return el('div', [], '<!-- Unsupported cell type -->');
            }
        },
        MarkdownCell: (cell, _notebook) => {
            return el('div', ['cell', 'markdown-cell'], renderMarkdown(joinText(cell.source)));
        },
        RawCell: (cell, _notebook) => {
            return el('div', ['cell', 'raw-cell'], joinText(cell.source));
        },
        CodeCell: (cell, notebook) => {
            const source = cell.source.length > 0
                ? r.Source(cell, notebook)
                : el('div');
            const outputs = coalesceStreams(cell.outputs || [])
                .map(output => r.Output(output, cell));
            return el('div', ['cell', 'code-cell'], [source, ...outputs]);
        },
        Source: (cell, notebook) => {
            const lang = notebookLanguage(notebook);
            const html = highlightCode(joinText(cell.source), lang);
            const codeEl = el('code', { 'class': `lang-${lang}`, 'data-language': lang }, html);
            const preEl = el('pre', [], [codeEl]);
            // Class "input" is for backward compatibility with notebook.js.
            const attrs = { ...executionCountAttrs(cell), class: 'source input' };
            return el('div', attrs, [preEl]);
        },
        Output: (output, cell) => {
            const innerEl = (() => {
                switch (output.output_type) {
                    case nbformat_1.OutputType.DisplayData: return r.DisplayData(output);
                    case nbformat_1.OutputType.ExecuteResult: return r.ExecuteResult(output);
                    case nbformat_1.OutputType.Stream: return r.Stream(output);
                    case nbformat_1.OutputType.Error: return r.Error(output);
                    default: return el('div', [], '<!-- Unsupported output type -->');
                }
            })();
            const attrs = { ...executionCountAttrs(cell), class: 'output' };
            return el('div', attrs, [innerEl]);
        },
        DisplayData: (output) => {
            const type = resolveDataType(output);
            if (type) {
                return dataRenderers[type](joinText(output.data[type]));
            }
            return el('div', ['empty-output']);
        },
        ExecuteResult: (output) => {
            const type = resolveDataType(output);
            if (type) {
                return dataRenderers[type](joinText(output.data[type]));
            }
            return el('div', ['empty-output']);
        },
        Error: (error) => {
            const html = renderAnsiCodes(error.traceback.join('\n'));
            // Class "pyerr" is for backward compatibility with notebook.js.
            return el('pre', ['error', 'pyerr'], html);
        },
        Stream: (stream) => {
            const html = renderAnsiCodes(joinText(stream.text));
            return el('pre', [stream.name], html);
        },
    });
    return r;
}
exports.default = buildRenderer;
// XXX: An ugly hack to infer return type of generic function that returns
// generalized object.
class DummyClass {
    constructor() {
        this.renderer = buildRenderer(this.elementCreator());
    }
}
//# sourceMappingURL=renderer.js.map